package bluetooth;
import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import java.util.Vector;
import java.util.Enumeration;
import javax.bluetooth.*;
import java.io.*;

public class Bluetooth extends MIDlet implements CommandListener
{
	Command CommandReconnectSerial = new Command("Reconnect serial", Command.OK, 1);
	Command CommandReconnectTCP = new Command("Reconnect TCP", Command.OK, 1);
	Command CommandStart = new Command("Start", Command.OK, 1);
	Command CommandExit = new Command("Exit", Command.OK, 1);
	
	/* Global */
	private Display display;
	private Form mainScreen;
	private StringItem resultField;
	private int SelectAction=0;
	private long FirstErrorTime=0;
	
	private boolean Reconnecting=true;
	private boolean StopThread=false;
	private Thread ComThreadThreadSer2TCP;				// thread
	private ComThreadSer2TCP ComThreadSer2TCPFunction;		// thread function

	private Thread ComThreadThreadTCP2Ser;				// thread
	private ComThreadTCP2Ser ComThreadTCP2SerFunction;		// thread function
	
	/* Serial */
	private NWSerial Serial=new NWSerial();
	private List DeviceSelectList = new List("Select device:", List.IMPLICIT);
	private List ServiceSelectList = new List("Select service:", List.IMPLICIT);
	private Vector ListDevices = new Vector();
	private Vector ListServices  = new Vector();
	private String ServiceURL;
	
	/* TCP */
	private NWTCP TCP = new NWTCP();
	
	// constructor
	public Bluetooth() 
	{
		display=Display.getDisplay(this);
		mainScreen=new Form("Mainscreen");
		mainScreen.addCommand(CommandStart);
		mainScreen.addCommand(CommandExit);
		mainScreen.setCommandListener(this);
		resultField = new StringItem(null, "");
		mainScreen.append(resultField);
	}
	
	// on start
	public void startApp() 
	{
		display.setCurrent(mainScreen);
		if(Serial.Init()==0)
		{
			AddMSG("Error init serial");
		}
		// start thread Ser2TCP
		ComThreadSer2TCPFunction=new ComThreadSer2TCP();
		ComThreadThreadSer2TCP=new Thread(ComThreadSer2TCPFunction);
		ComThreadThreadSer2TCP.start();
		ComThreadThreadSer2TCP.setPriority(Thread.MIN_PRIORITY);
		// start thread TCP2Ser
		ComThreadTCP2SerFunction=new ComThreadTCP2Ser();
		ComThreadThreadTCP2Ser=new Thread(ComThreadTCP2SerFunction);
		ComThreadThreadTCP2Ser.start();
		ComThreadThreadTCP2Ser.setPriority(Thread.MIN_PRIORITY);
	}

	// on pause
	public void pauseApp() 
	{
		
	}

	// on destroy
	public void destroyApp(boolean unconditional) 
	{
		
	}

	// on exit
	public void exitMIDlet() 
	{
		destroyApp(true);
		notifyDestroyed();
	}
	
	private void AddMSG(String Message)
	{
		resultField.setText(resultField.getText()+"\n"+Message);
		//resultField.setText(Message);
	}
	
	// interpret the commands: exit, list select, ... 
	public void commandAction(Command c, Displayable s)
	{
		if(c == CommandExit)
		{
			destroyApp(false);
			notifyDestroyed();
		}
		else if(c == CommandStart)
		{
			mainScreen.removeCommand(CommandStart);
			mainScreen.removeCommand(CommandExit);
			SelectDevice();
		}
		else if(c == List.SELECT_COMMAND)  // device or service selected
		{
			display.setCurrent(mainScreen);
			mainScreen.addCommand(CommandExit);
			if(SelectAction==0) // device selected
			{
				AddMSG("DeviceSelected()");
				DeviceSelected();
			}
			else if(SelectAction==1) // service select
			{
				AddMSG("ServiceSelected()");
				ServiceSelected();
			}
		}
		else if(c == CommandReconnectSerial)
		{
			SerialReconnect();
		}
		else if(c == CommandReconnectTCP)
		{
			TCPReconnect();
		}
	}
	
    public int SelectDevice()
    {
		if(Serial.SearchForDevices()==0)
			AddMSG("Error searching for devices");
		while(!Serial.DeviceSearchFinished())
			try { Thread.sleep(100); } catch(InterruptedException Error) { }
		if(Serial.GetDeviceCout()==0)
		{
			AddMSG("Found no devices...");
			return 0;
		}
		ListDevices=Serial.GetDevices();
		AddMSG("Devices found: "+ListDevices.size());
		Enumeration EnumDevices = ListDevices.elements();
		RemoteDevice SingleDevice;
		String DeviceName;
		DeviceSelectList.deleteAll();
		while(EnumDevices.hasMoreElements())
		{
			SingleDevice=(RemoteDevice)EnumDevices.nextElement();
			try
			{
				DeviceName=SingleDevice.getFriendlyName(false);
			}
			catch(IOException Error)
			{
				DeviceName=SingleDevice.getBluetoothAddress();
			}
			DeviceSelectList.append(DeviceName, null);
		}
		SelectAction=0;	// on select event, use the code for device selection
		DeviceSelectList.setCommandListener(this);
		display.setCurrent(DeviceSelectList);
		return 1;
    }
	
	private int DeviceSelected()
	{
		AddMSG("selected nr: "+DeviceSelectList.getSelectedIndex());
		RemoteDevice SingleDevice=(RemoteDevice)ListDevices.elementAt(DeviceSelectList.getSelectedIndex());
		String Name;
		try
		{
			Name=SingleDevice.getFriendlyName(false);
		}
		catch(IOException Error)
		{
			Name=SingleDevice.getBluetoothAddress();
		}
		AddMSG("Selected: "+Name);
		// now search for services
		Serial.SearchForServices(SingleDevice);
		while(!Serial.ServiceSearchFinished())
			try { Thread.sleep(100); } catch(InterruptedException Error) { }
		if(Serial.GetServiceCount()==0)
		{
			AddMSG("Found no services...");
			return 0;
		}
		ListServices=Serial.GetServices();
		Enumeration EnumServices = ListServices.elements();
		ServiceRecord SingleService;
		ServiceSelectList.deleteAll();
		while(EnumServices.hasMoreElements())
		{
			SingleService=(ServiceRecord)EnumServices.nextElement();
			DataElement serviceNameElement = SingleService.getAttributeValue(0x0100);
			String serviceName = (String)serviceNameElement.getValue();
			ServiceSelectList.append(serviceName, null);
		}
		SelectAction=1;
		ServiceSelectList.setCommandListener(this);
		display.setCurrent(ServiceSelectList);
		return 1;
	}
	
	private int ServiceSelected()
	{
		// save service URL
		AddMSG("Selected a service..."); 
		ServiceRecord service=(ServiceRecord)ListServices.elementAt(ServiceSelectList.getSelectedIndex());
		ServiceURL=service.getConnectionURL(ServiceRecord.NOAUTHENTICATE_NOENCRYPT, false);
		//Serial.Disconnect();
		Serial.Connect(ServiceURL);
		mainScreen.addCommand(CommandReconnectSerial);
		mainScreen.addCommand(CommandReconnectTCP);
		//mainScreen.addCommand(CommandSelectDevice);
		display.setCurrent(mainScreen);
		
		// serial connected, now connect tcp
		if(TCP.Connect()==0)
		{
			AddMSG("Error connecting TCP!");
			return 0;
		}
		AddMSG("TCP init...");
		Reconnecting=false;
		return 1;
	}
	
	private void SerialReconnect()
	{
		FirstErrorTime=System.currentTimeMillis();
		Reconnecting=true;
		int ReturnValue;
		ReturnValue=Serial.Reconnect();
		while(ReturnValue<1)
		{
			if(ReturnValue==0)
				AddMSG("Error reconnecting");
			if(ReturnValue==-1)
				AddMSG("No URL!");
			if(System.currentTimeMillis()-FirstErrorTime>30000)
			{
				GlobalAlert();
				return;
			}
			ReturnValue=Serial.Reconnect();
		}
		Reconnecting=false;
		AddMSG("Reconnected serial");
	}
	
	private void TCPReconnect()
	{
		FirstErrorTime=System.currentTimeMillis();
		Reconnecting=true;
		int ReturnValue;
		ReturnValue=TCP.Reconnect();
		while(ReturnValue<1)
		{
			if(ReturnValue==0)
				AddMSG("Error reconnecting");
			if(System.currentTimeMillis()-FirstErrorTime>30000)
			{
				GlobalAlert();
				return;
			}
			ReturnValue=TCP.Reconnect();
		}
		Reconnecting=false;
		AddMSG("Reconnected TCP");
	}
	
	private class ComThreadSer2TCP implements Runnable
	{
		byte[] Data=new byte[1000];
		
		public final void run()
		{
			while(!StopThread)
			{
				while(Reconnecting) // don't transport data when we are reconnecting or not connected
					try { Thread.sleep(100); } catch(InterruptedException Error) { }
				
				int ByteCount=Serial.GetData(Data);
				if(ByteCount>0)
				{
					if(TCP.SendData(Data, ByteCount)==0)
					{
						AddMSG("Error sending data on tcp");
						TCPReconnect();
					}
				}
				else if(ByteCount<0)
				{
					AddMSG("Error getting data on serial");
					SerialReconnect();
				}
			}
			return;
		}
	}

	private class ComThreadTCP2Ser implements Runnable
	{
		byte[] Data=new byte[1000];
		
		public final void run()
		{
			while(!StopThread)
			{
				while(Reconnecting) // don't transport data when we are reconnecting or not connected
					try { Thread.sleep(100); } catch(InterruptedException Error) { }
				
				int ByteCount=TCP.GetData(Data);
				if(ByteCount>0)
				{
					if(Serial.SendData(Data, ByteCount)==0)
					{
						AddMSG("Error sending data on serial");
						SerialReconnect();
					}
				}
				else if(ByteCount<0)
				{
					AddMSG("Error getting data on tcp");
					TCPReconnect();
				}
			}
			return;
		}
	}
	
	private void GlobalAlert()
	{
		AddMSG("Global alert!");
	}
}